/*
    AttendanceManager
    Copyright (C) 2010  Victor Carceler

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU Affero General Public License as
    published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Affero General Public License for more details.

    You should have received a copy of the GNU Affero General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>
 */

package org.iespuigcastellar.attendancemanager.core;

import java.sql.Time;
import org.iespuigcastellar.attendancemanager.AttendancemanagerApplication;

/**
 * Model a database classblock record. Has all record columns plus a name
 * property used by the user interface. Name is not stored on DBMS.
 * 
 * TODO: método initName para calcular el name a partir de la BBDD.
 * 
 * @author vcarceler
 */
public class Classblock implements Entity, HasGetName, HasGetID {


	private int id;
	private int idsubject;
	private int idgroup;
	private int idterm;
	private int idteacher;
	private int day_of_week;
	private Time start;
	private Time end;
	private String name = ""; // It's a name composed by time + group.name + " / " + subject.name
	
	
	private static String daynames[] = {AttendancemanagerApplication.locale.getString("SATURDAY"), 
										AttendancemanagerApplication.locale.getString("SUNDAY"), 
										AttendancemanagerApplication.locale.getString("MONDAY"),
										AttendancemanagerApplication.locale.getString("TUESDAY"), 
										AttendancemanagerApplication.locale.getString("WEDNESDAY"), 
										AttendancemanagerApplication.locale.getString("THURSDAY"), 
										AttendancemanagerApplication.locale.getString("FRIDAY")};
	

	public Classblock(int id, int idsubject, int idgroup, int day_of_week, Time start, Time end) {
		this.id = id;
		this.idsubject = idsubject;
		this.idgroup = idgroup;
		this.day_of_week = day_of_week;
		this.start = start;
		this.end = end;
	}

	public Classblock(int id, int idterm, int idteacher, int idsubject, int idgroup, int day_of_week, Time start, Time end) {
		this(id, idsubject, idgroup, day_of_week, start, end);

		this.idterm = idterm;
		this.idteacher = idteacher;		
	}
	
	// És un constructor de tipus Classblock que comproba que tots els elemnts dels classblock estigui en la base de dades storage y retorni els seus ids. 
	public Classblock(String day_of_week, String hora, String idgroup, String idsubject, String idteacher) {
		this(0, 1, storage.getIDTeacher(idteacher), storage.getIDSubject(idsubject), 
				storage.getIDGroup(idgroup), getDayName(day_of_week), toTime(hora,0), toTime(hora,1));
	}
	
	public int getIDgroup() {
		return idgroup;
	}

	public void setIDgroup(int idgroup) {
		this.idgroup = idgroup;
	}

	public int getIDTerm() {
		return idterm;
	}

	public void setIDTerm(int idterm) {
		this.idterm = idterm;
	}

	public int getIDTeacher() {
		return idteacher;
	}

	public void setIDTeacher(int idteacher) {
		this.idteacher = idteacher;
	}
	public int getID() {
		return id;
	}
	public void setID(int id) {
		this.id = id;
	}
	public int getIDSubject() {
		return idsubject;
	}
	public void setIDSubject(int idsubject) {
		this.idsubject = idsubject;
	}
	public int getDay_of_week() {
		return day_of_week;
	}
	public void setDay_of_week(int dayOfWeek) {
		day_of_week = dayOfWeek;
	}
	public Time getStart() {
		return start;
	}
	public void setStart(Time start) {
		this.start = start;
	}
	public Time getEnd() {
		return end;
	}
	public void setEnd(Time end) {
		this.end = end;
	}

	public void setIDGroup(int idgroup) {
		this.idgroup = idgroup;
	}

	public void setName(String name) {
		this.name = name;
	}

	public String getName() {
		return name;
	}

	@Override
	public String toString() {
		return "Classblock [day_of_week=" + day_of_week + ", end=" + end
				+ ", id=" + id + ", idgroup=" + idgroup + ", idsubject="
				+ idsubject + ", name=" + name + ", start=" + start + "]";
	}
	
	public String getDayName() {
		return daynames[day_of_week];
	}
	
	public static String getDayName(int day_of_week) {
		return daynames[day_of_week];
	}
	
	
	/**
	 * Agafem el dia de la setmana del classblock i la comparem amb els dies que esta enmagatzemades a la base de dades i retornem un valor.
	 * @param day_of_week
	 * @return res
	 */
	public static int getDayName(String day_of_week) {
		int res=0;
		for (int i=0;i<daynames.length;i++){
			if (daynames[i].toString().compareTo(day_of_week)==0){
				return i;
			}		
		}
		return res;
	}
	
	
	/**
	 * Aquesta funció serveix per llegir la capçalera dels fitxer csv i que ha de tenir aquest ordre.
	 * @return columns
	 */
	public static String[] getBasicProperties() {
		String columns[] =  {"Day","Period","Students Sets","Subject","Teachers"};
		return columns;
	}
	

	/**
	 * Se añade él mismo a la base de datos
	 * @return el id del Subject insertado
	 */
	public int add() {
		return storage.addClassblock(this);
	}

	/**
	 * Se elimina él mismo de la base de datos
	 */
	public void delete() {
		 storage.deleteClassblock(this.id);
	}

	/**
	 * Se modifica él mismo en la base de datos
	 * @return el código de retorno
	 */
	public int update() {
		return storage.updateClassblock(this);
	}	
	
	/**
	 * Crea un objeto de tipo Time para añadir al Classblock a partir de un String introducido como parametro. 
	 * En funcion del valor del entero que se pasa como parametro, se devuelve la hora para 'start' o 'end'.
	 * @param hora
	 * @param i
	 * @return
	 */
	public static Time toTime(String hora, int i) {
		if (i==0){
			String h = hora.substring(0, 2),m = hora.substring(3, 5);
			Logger.log("start: h = " + h +" ,m = " + m);
			Time start = new Time(Integer.parseInt(h), Integer.parseInt(m), 00);
			return start;
		}
		else {
			String h = hora.substring(6, 8),m = hora.substring(9, 11);
			Logger.log("end: h = " + h +" ,m = " + m);
			Time end = new Time(Integer.parseInt(h), Integer.parseInt(m), 00);
			return end;
		}
	}
}