/*
    AttendanceManager
    Copyright (C) 2010  Victor Carceler

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU Affero General Public License as
    published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Affero General Public License for more details.

    You should have received a copy of the GNU Affero General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>
 */

package org.iespuigcastellar.attendancemanager.screenlayouts;

import java.sql.Time;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.ResourceBundle;

import org.iespuigcastellar.attendancemanager.AttendancemanagerApplication;
import org.iespuigcastellar.attendancemanager.core.CSVUploader;
import org.iespuigcastellar.attendancemanager.core.Classblock;
import org.iespuigcastellar.attendancemanager.core.DatabaseStorage;
import org.iespuigcastellar.attendancemanager.core.Group;
import org.iespuigcastellar.attendancemanager.core.Subject;
import org.iespuigcastellar.attendancemanager.core.Teacher;
import org.iespuigcastellar.attendancemanager.core.Term;

import com.vaadin.data.Item;
import com.vaadin.data.Property;
import com.vaadin.data.Property.ValueChangeEvent;
import com.vaadin.data.util.IndexedContainer;
import com.vaadin.data.util.QueryContainer;
import com.vaadin.terminal.ThemeResource;
import com.vaadin.ui.Alignment;
import com.vaadin.ui.Button;
import com.vaadin.ui.Button.ClickEvent;
import com.vaadin.ui.ComboBox;
import com.vaadin.ui.CssLayout;
import com.vaadin.ui.Embedded;
import com.vaadin.ui.GridLayout;
import com.vaadin.ui.HorizontalLayout;
import com.vaadin.ui.Label;
import com.vaadin.ui.NativeSelect;
import com.vaadin.ui.Panel;
import com.vaadin.ui.Table;
import com.vaadin.ui.TextField;
import com.vaadin.ui.Upload;
import com.vaadin.ui.Upload.FailedEvent;
import com.vaadin.ui.Upload.SucceededEvent;
import com.vaadin.ui.VerticalLayout;
import com.vaadin.ui.Window;

@SuppressWarnings("serial")
public class ClassblockManagerLayout extends VerticalLayout{
	private AttendancemanagerApplication app;
	
	private static ResourceBundle locale = AttendancemanagerApplication.locale;
	private static DatabaseStorage storage = AttendancemanagerApplication.storage;
	
	private Button newButton;
	private Button editButton;
	private Button deleteButton;
	
	//Creem dos nous botons que seran el de importar i el de buscars.
	private Button importButton;
	private Button searchButton;
	
	private Table classblocksTable;
	
	private NewClassblockWindow newClassblockWindow;
	private EditClassblockWindow editClassblockWindow;
	private DeleteClassblockWindow deleteClassblockWindow;
	
	//Creem dos finestres. Un per importar blocslectius i l'altre per buscar l'horari d'un professor.
	private ImportClassblockWindow importClassblockWindow;
	private SearchClassblockWindow searchClassblockWindow;
	
	private void initLayout() {
		newButton = new Button(locale.getString("GROUPMANAGERLAYOUT_NEWBUTTON_CAPTION"));
		newButton.setIcon(new ThemeResource("img/clock_add.png"));
		newButton.setStyleName("default");
		
	
		editButton = new Button(locale.getString("GROUPMANAGERLAYOUT_EDITBUTTON_CAPTION"));
		editButton.setIcon(new ThemeResource("img/clock_edit.png"));
		editButton.setEnabled(false);
		editButton.setStyleName("default");
		
		deleteButton = new Button(locale.getString("GROUPMANAGERLAYOUT_DELETEBUTTON_CAPTION"));
		deleteButton.setIcon(new ThemeResource("img/clock_delete.png"));
		deleteButton.setEnabled(false);
		deleteButton.setStyleName("default");
		
		// Al botó importar li afegim el nom que surtirà per pantalla, li donem una imtge que surtirà quan es visualitzi el botó i el un tema per defecte.
		importButton = new Button(locale.getString("GROUPMANAGERLAYOUT_IMPORTBUTTON_CAPTION"));
		importButton.setIcon(new ThemeResource("img/clock_go.png"));
		importButton.setStyleName("default");

		// Al botó buscar li afegim el nom que surtirà per pantalla, li donem una imtge que surtirà quan es visualitzi el botó i el un tema per defecte.
		searchButton = new Button(locale.getString("GROUPMANAGERLAYOUT_SEARCHBUTTON_CAPTION"));
		searchButton.setIcon(new ThemeResource("img/magnifier.png"));
		searchButton.setStyleName("default");
		
		CssLayout toolbarclassblock = new CssLayout();
		toolbarclassblock.addComponent(newButton);
		toolbarclassblock.addComponent(editButton);
		toolbarclassblock.addComponent(deleteButton);
		
		// Afegim els botons al toolbarclassblock
		toolbarclassblock.addComponent(importButton);
		toolbarclassblock.addComponent(searchButton);
		
		toolbarclassblock.setStyleName("toolbar");
		toolbarclassblock.setWidth("100%");
		
		classblocksTable = new Table();
		classblocksTable.setImmediate(true);
		classblocksTable.setColumnReorderingAllowed(true);
		classblocksTable.setColumnCollapsingAllowed(true);
		classblocksTable.setStyleName("striped");
		classblocksTable.setSizeFull();
				

		Panel subjectsPanel = new Panel(locale.getString("CLASSBLOCKMANAGERLAYOUT_CLASSBLOCKPANEL_CAPTION"));
		subjectsPanel.addComponent(classblocksTable);
		subjectsPanel.setSizeFull();
		subjectsPanel.setStyleName("bubble");
		subjectsPanel.setIcon(new ThemeResource("img/clock.png"));
		subjectsPanel.getContent().setSizeFull();
			
		
		VerticalLayout vl =  (VerticalLayout) subjectsPanel.getContent();
		vl.setMargin(false);
		vl.setExpandRatio(classblocksTable, 1.0f);
		vl.setSpacing(false);
		
		addComponent(toolbarclassblock);
		addComponent(subjectsPanel);
		
		setSizeFull();
		setMargin(false, true, false, true);
		
		setExpandRatio(subjectsPanel, 1.0f);
		setSpacing(true);
	}
	
	public void attach() {

		app = (AttendancemanagerApplication)getApplication();
		initLayout();
		classblocksTable.addContainerProperty(DatabaseStorage.TEACHER_IDTEACHER_PROPERTY, Integer.class, "");
		classblocksTable.addContainerProperty(DatabaseStorage.SUBJECT_IDSUBJECT_PROPERTY, Integer.class, "");
		classblocksTable.addContainerProperty(DatabaseStorage.GROUP_IDGROUP_PROPERTY, Integer.class, "");
		classblocksTable.addContainerProperty(DatabaseStorage.TERM_IDTERM_PROPERTY, Integer.class, "");
		classblocksTable.addContainerProperty(DatabaseStorage.TEACHERNAME_PROPERTY, String.class, "");
		classblocksTable.addContainerProperty(DatabaseStorage.TERMNAME_PROPERTY, String.class, "");
		classblocksTable.addContainerProperty(DatabaseStorage.GROUPNAME_PROPERTY, String.class, "");
		classblocksTable.addContainerProperty(DatabaseStorage.SUBJECTNAME_PROPERTY, String.class, "");
		classblocksTable.addContainerProperty(DatabaseStorage.START_PROPERTY, String.class, "");
		classblocksTable.addContainerProperty(DatabaseStorage.END_PROPERTY, String.class, "");
		classblocksTable.addContainerProperty(DatabaseStorage.DAYOFWEEKSTRING_PROPERTY, String.class, "");
		classblocksTable.addContainerProperty(DatabaseStorage.DAYOFWEEK_PROPERTY, Integer.class, "");
		String visiblecolumns[]={
				DatabaseStorage.TEACHERNAME_PROPERTY,
				DatabaseStorage.TERMNAME_PROPERTY,
				DatabaseStorage.GROUPNAME_PROPERTY,
				DatabaseStorage.SUBJECTNAME_PROPERTY,
				DatabaseStorage.START_PROPERTY,
				DatabaseStorage.END_PROPERTY,
				DatabaseStorage.DAYOFWEEKSTRING_PROPERTY
			};
		
		
		classblocksTable.setVisibleColumns(visiblecolumns);
		classblocksTable.setColumnHeaders(new String[] {
				locale.getString("CLASSBLOCKMANAGERLAYOUT_CLASSBLOCKTABLE_TEACHERNAME"),
				locale.getString("CLASSBLOCKMANAGERLAYOUT_CLASSBLOCKTABLE_TERMNAME"),
				locale.getString("CLASSBLOCKMANAGERLAYOUT_CLASSBLOCKTABLE_GROUPNAME"),
				locale.getString("CLASSBLOCKMANAGERLAYOUT_CLASSBLOCKTABLE_SUBJECTNAME"),
				locale.getString("TERMMANAGERLAYOUT_TERMSTABLE_STARTCOLUMN"),
				locale.getString("TERMMANAGERLAYOUT_TERMSTABLE_ENDCOLUMN"),
				locale.getString("CLASSBLOCKMANAGERLAYOUT_CLASSBLOCKTABLE_DAYOFWEEK")
				});
		classblocksTable.setSelectable(true);
		classblocksTable.setMultiSelect(false);
		classblocksTable.setImmediate(true);
		classblocksTable.addListener(new Property.ValueChangeListener() {
			
			@Override
			public void valueChange(ValueChangeEvent event) {
				Object value = classblocksTable.getValue();
				if (value!=null){
					editButton.setEnabled(true);
					deleteButton.setEnabled(true);
				} else {
					editButton.setEnabled(false);
					deleteButton.setEnabled(false);
					
				}
			}
		});
	    newClassblockWindow = new NewClassblockWindow();
	    newClassblockWindow.setCaption(locale.getString("CLASSBLOCKMANAGERLAYOUT_NEWCLASSBLOCKWINDOW_CAPTION"));
		newButton.addListener(new Button.ClickListener() {
			
			@Override
			public void buttonClick(ClickEvent event) {
				// Open window if not open already
				if(newClassblockWindow.getParent() != null) {
					// Window already open
				} else {
					// refresh Combos (Teachers,Terms.....)
					newClassblockWindow.refreshCombos();						
					getWindow().addWindow(newClassblockWindow);
				}
			}
		});
		editClassblockWindow = new EditClassblockWindow();
		editClassblockWindow.setCaption(locale.getString("CLASSBLOCKMANAGERLAYOUT_EDITCLASSBLOCKWINDOW_CAPTION"));
		editButton.addListener(new Button.ClickListener() {
			
			@Override
			public void buttonClick(ClickEvent event) {

				editClassblockWindow.refreshCombos();
				Integer idItem=(Integer) classblocksTable.getValue();
				Item item=classblocksTable.getItem(idItem);
				Integer idClassblock=(Integer) classblocksTable.getValue();
				Integer dayofweek=(Integer) item.getItemProperty(DatabaseStorage.DAYOFWEEK_PROPERTY).getValue();
				Integer idteacher=(Integer) item.getItemProperty(DatabaseStorage.TEACHER_IDTEACHER_PROPERTY).getValue();
				Integer idgroup=(Integer) item.getItemProperty(DatabaseStorage.GROUP_IDGROUP_PROPERTY).getValue();
				Integer idsubject=(Integer) item.getItemProperty(DatabaseStorage.SUBJECT_IDSUBJECT_PROPERTY).getValue();
				Integer idterm=(Integer) item.getItemProperty(DatabaseStorage.TERM_IDTERM_PROPERTY).getValue();
				//SimpleDateFormat sdf = new SimpleDateFormat("HH:mm");
				Time start=	Time.valueOf((String) item.getItemProperty(DatabaseStorage.START_PROPERTY).getValue());
				Time end=	Time.valueOf((String) item.getItemProperty(DatabaseStorage.END_PROPERTY).getValue());
				
				Classblock classblock = new Classblock(idClassblock,idterm,idteacher, idsubject, idgroup, dayofweek, start, end);
				editClassblockWindow.setClassblock(classblock);
				getWindow().addWindow(editClassblockWindow);
			}
		});	
		deleteClassblockWindow = new DeleteClassblockWindow();
		deleteClassblockWindow.setCaption(locale.getString("CLASSBLOCKMANAGERLAYOUT_DELETECLASSBLOCKWINDOW_CAPTION"));
		deleteButton.addListener(new Button.ClickListener() {
			
			@Override
			public void buttonClick(ClickEvent event) {

				//Integer idItem=(Integer) classblocksTable.getValue();
				//Item item=classblocksTable.getItem(idItem);
				Integer idClassblock=(Integer) classblocksTable.getValue();
				deleteClassblockWindow.setIdClassblock(idClassblock);
				getWindow().addWindow(deleteClassblockWindow);
			}
		});
		
		
		// Es el constructor del del importClassblockWindow
		try {
			importClassblockWindow = new ImportClassblockWindow();
		} catch (SecurityException e) {

			e.printStackTrace();
			
		} catch (NoSuchMethodException e) {
			
			e.printStackTrace();
		}
		
		// Mostra un etiqueta amb la informació del botó. Controla que quan es cliqui el botó mostri la finestra d'importar.
		importClassblockWindow.setCaption(locale.getString("CLASSBLOCKMANAGERLAYOUT_IMPORTCLASSBLOCKWINDOW_CAPTION"));
		importButton.addListener(new Button.ClickListener() {
			
			@Override
			public void buttonClick(ClickEvent event) {

				getWindow().addWindow(importClassblockWindow);
			}
		}); 
		
		// Es la classe constructora de searchClassblockWindow.
		searchClassblockWindow = new SearchClassblockWindow();
		
		// Mostra un etiqueta amb la informació del botó. Controla que quan es cliqui el botó mostri la finestra de buscar.
		searchClassblockWindow.setCaption(locale.getString("CLASSBLOCKMANAGERLAYOUT_SEACRHCLASSBLOCKWINDOW_CAPTION"));
		searchButton.addListener(new Button.ClickListener() {
			
			@Override
			public void buttonClick(ClickEvent event) {
				// Open window if not open already
				if(searchClassblockWindow.getParent() != null) {
					// Window already open
				} else {
					// refresh Combos (Teachers,Terms.....)	
					searchClassblockWindow=new SearchClassblockWindow();
					getWindow().addWindow(searchClassblockWindow);
				}
			}
		});
	}
	
	private boolean isValid (){
		
		//IF combo teachers is empty or the rest you can not add new classblock
		if (newClassblockWindow.combos.isEmptyGroups() ||
			newClassblockWindow.combos.isEmptyTeachers() ||
			newClassblockWindow.combos.isEmptyTerms() ||
			newClassblockWindow.combos.isEmptySubjects()){
			
			return false;
		}
		
		return true;		
	}
	
	public void refreshLayout() {
		storage.refreshTable(storage.getClassblocks(), classblocksTable);
		newClassblockWindow.refreshCombos();
		editClassblockWindow.refreshCombos();
		if (! isValid()){ 
			newButton.setEnabled(false);
		}else{
			newButton.setEnabled(true);
		}
	}	
	
	private class NewClassblockWindow extends Window {
		
		public final ClassBlockLayout combos = new ClassBlockLayout();
		
		public NewClassblockWindow() {
			
			Button okButton = new Button(locale.getString("GROUPMANAGERLAYOUT_BUTTONOK_CAPTION"));
			okButton.addListener(new Button.ClickListener() {		
				@Override
			public void buttonClick(ClickEvent event) {
					Classblock classblock = new Classblock(
								0,
								combos.getSelectTerm(),
								combos.getSelectedTeacher(),
								combos.getSelectedSubject(),
								combos.getSelectedGroup(),
								combos.getSelectedDayofweek(),
								combos.getTimeStart(),
								combos.getTimeEnd()
						);
					Integer newclassblock =storage.addClassblock(classblock);
					getParent().getWindow().showNotification(
							locale.getString("GROUPMANAGERLAYOUT_TOOLTIP_MESSAGE"),
	                    	locale.getString("CLASSBLOCKMANAGERLAYOUT_TOOLTIP_CLASSBLOCKADDED"),
                            Notification.TYPE_TRAY_NOTIFICATION);
					refreshLayout();
					setValuesDefault();
					classblocksTable.setValue(newclassblock);
					close();
				}
			});
			okButton.setIcon(new ThemeResource("img/12-em-check.png"));
			Button cancelButton = new Button(locale.getString("GROUPMANAGERLAYOUT_BUTTONCANCEL_CAPTION"));
			cancelButton.addListener(new Button.ClickListener() {
				
				@Override
				public void buttonClick(ClickEvent event) {   
					setValuesDefault();
                	close();
				}
			});
			cancelButton.setIcon(new ThemeResource("img/12-em-cross.png"));
			
			HorizontalLayout hl = new HorizontalLayout();
			hl.setSpacing(true);
			hl.addComponent(okButton);
			hl.addComponent(cancelButton);
			
			VerticalLayout layouts= new VerticalLayout();
			layouts.setSpacing(true);
			layouts.addComponent(combos);
			layouts.addComponent(hl);
    		
    		VerticalLayout layout = (VerticalLayout) getContent();
    		layout.setSpacing(true);
    		
    		 // We want the window to has automatic size !!!
    		layout.addComponent(layouts);
    		layout.setSizeUndefined();
    		setModal(true);
    		setResizable(false);

		}
		
		private void setValuesDefault(){
			
		}
		
		private void refreshCombos(){
			combos.loadGroups();
			combos.loadTeachers();
			combos.loadTerms();
			combos.loadSubjects();
			combos.loadDaysofWeek();
			combos.loadHours();
			combos.loadMinutes();
		}
	}
	
	private class EditClassblockWindow extends Window {
		
		public final ClassBlockLayout combos = new ClassBlockLayout();
		private Classblock classblock;
		public EditClassblockWindow() {
			
			Button okButton = new Button(locale.getString("GROUPMANAGERLAYOUT_BUTTONOK_CAPTION"));
			okButton.addListener(new Button.ClickListener() {		
				@Override
			public void buttonClick(ClickEvent event) {
					classblock.setIDTeacher(combos.getSelectedTeacher());
					classblock.setIDTerm(combos.getSelectTerm());
					classblock.setIDGroup(combos.getSelectedGroup());
					classblock.setIDSubject(combos.getSelectedSubject());
					classblock.setDay_of_week(combos.getSelectedDayofweek());
					classblock.setStart(combos.getTimeStart());
					classblock.setEnd(combos.getTimeEnd());
					if (storage.updateClassblock(classblock)!=0) {
						getParent().getWindow().showNotification(
								locale.getString("GROUPMANAGERLAYOUT_TOOLTIP_MESSAGE"),
	                    		locale.getString("CLASSBLOCKMANAGERLAYOUT_TOOLTIP_CLASSBLOCKEDITED"),
	                            Notification.TYPE_TRAY_NOTIFICATION);
					};
					refreshLayout();
					classblocksTable.setValue(classblock.getID());
					close();
				}
			});
			okButton.setIcon(new ThemeResource("img/12-em-check.png"));
			Button cancelButton = new Button(locale.getString("GROUPMANAGERLAYOUT_BUTTONCANCEL_CAPTION"));
			cancelButton.addListener(new Button.ClickListener() {
				
				@Override
				public void buttonClick(ClickEvent event) {   
				
                	close();
				}
			});
			cancelButton.setIcon(new ThemeResource("img/12-em-cross.png"));
			
			HorizontalLayout hl = new HorizontalLayout();
			hl.setSpacing(true);
			hl.addComponent(okButton);
			hl.addComponent(cancelButton);
			
			VerticalLayout layouts= new VerticalLayout();
			layouts.setSpacing(true);
			layouts.addComponent(combos);
			layouts.addComponent(hl);
    		
    		VerticalLayout layout = (VerticalLayout) getContent();
    		layout.setSpacing(true);
    		
    		 // We want the window to has automatic size !!!
    		layout.addComponent(layouts);
    		layout.setSizeUndefined();
    		setModal(true);
    		setResizable(false);

		}
		private void refreshCombos(){
			combos.loadGroups();
			combos.loadTeachers();
			combos.loadTerms();
			combos.loadSubjects();
			combos.loadDaysofWeek();
			combos.loadHours();
			combos.loadMinutes();
		}
		private void setClassblock(Classblock classblock){
			combos.setTeacher(classblock.getIDTeacher());
			combos.setTerm(classblock.getIDTerm());
			combos.setGroup(classblock.getIDgroup());
			combos.setSubject(classblock.getIDSubject());
			combos.setDayofweek(classblock.getDay_of_week());
			combos.setHourStart(classblock.getStart());
			combos.setHourEnd(classblock.getEnd());
			this.classblock=classblock;
		}
	}	
	private class DeleteClassblockWindow extends Window{
		
		//private Subject subject;
		private Label messageLabel;
		private Integer idClassblock=0;
		public DeleteClassblockWindow() {
			
			Embedded warningEmbedded = new Embedded(null,new ThemeResource("img/Warning_icon.png"));
			 
			messageLabel = new Label();
			String messageString = locale.getString("CLASSBLOCKMANAGERLAYOUT_DELETEWINDOW_MESSAGE");
			
			messageLabel.setValue(messageString);
			messageLabel.setContentMode(Label.CONTENT_XHTML);
			
			HorizontalLayout warningLayout = new HorizontalLayout();
			warningLayout.setSpacing(true);
			warningLayout.addComponent(warningEmbedded);
			warningLayout.addComponent(messageLabel);
			warningLayout.setSizeFull();
			Button okButton = new Button(locale.getString("GROUPMANAGERLAYOUT_BUTTONOK_CAPTION"));
			okButton.addListener(new Button.ClickListener() {
				
				@Override
				public void buttonClick(ClickEvent event) {
					storage.deleteClassblock(idClassblock);
                    getParent().getWindow().showNotification(
                    		locale.getString("GROUPMANAGERLAYOUT_TOOLTIP_MESSAGE"),
                    		locale.getString("CLASSBLOCKMANAGERLAYOUT_TOOLTIP_CLASSBLOCKDELETED"),
                            Notification.TYPE_TRAY_NOTIFICATION);
                    refreshLayout();
					close();
				}
			});
			okButton.setIcon(new ThemeResource("img/12-em-check.png"));
			
			Button cancelButton = new Button(locale.getString("GROUPMANAGERLAYOUT_BUTTONCANCEL_CAPTION"));
			cancelButton.addListener(new Button.ClickListener() {
				
				@Override
				public void buttonClick(ClickEvent event) {
					close();
				}
			});
			cancelButton.setIcon(new ThemeResource("img/12-em-cross.png"));
			HorizontalLayout hl = new HorizontalLayout();
			hl.setSpacing(true);
			hl.setMargin(true);
			hl.addComponent(okButton);
			hl.addComponent(cancelButton);
			hl.setSizeFull();
			addComponent(warningLayout);
    		addComponent(hl);
    		
    		VerticalLayout layout = (VerticalLayout) getContent();
    		layout.setSpacing(true);
    		layout.setMargin(true);
    		layout.setSizeUndefined(); // We want the window to has automatic size !!!
    		
    		setModal(true);
    		setResizable(false);
		}

		private void setIdClassblock(Integer idClassblock){
			   this.idClassblock=idClassblock;
		}
	}
	public class  ClassBlockLayout extends VerticalLayout{
		private NativeSelect dayofweek;
		private NativeSelect hourStart;
		private NativeSelect minuteStart;
		private NativeSelect hourEnd;
		private NativeSelect minuteEnd;
		private ComboBox teachers;
		private ComboBox subjects;
		private ComboBox groups;
		private ComboBox terms;
		
		public ClassBlockLayout(){
			setMargin(true);
			GridLayout gl = new GridLayout();
			gl.setRows(8);
			gl.setColumns(10);
			
			HorizontalLayout starttimeLayout = new HorizontalLayout();
			HorizontalLayout endtimeLayout = new HorizontalLayout();
			Label labelStart=new Label(locale.getString("CLASSBLOCKMANAGERLAYOUT_LABELSTART_CAPTION"));
			Label labelEnd=new Label(locale.getString("CLASSBLOCKMANAGERLAYOUT_LABELEND_CAPTION"));
			
			Label pointsLabel = new Label("<h3>:</h3>",Label.CONTENT_XML);
			Label pointsLabel2 = new Label("<h3>:</h3>",Label.CONTENT_XML);
			
			hourStart=new NativeSelect();
			hourStart.setNullSelectionAllowed(false);
			minuteStart=new NativeSelect();
			minuteStart.setNullSelectionAllowed(false);
			dayofweek=new NativeSelect(locale.getString("CLASSBLOCKMANAGERLAYOUT_CLASSBLOCKTABLE_DAYOFWEEK"));
			dayofweek.setImmediate(true);
			dayofweek.setNullSelectionAllowed(false);
			dayofweek.setInvalidAllowed(false);
			dayofweek.setReadThrough(true);
			dayofweek.setSizeFull();
			starttimeLayout.addComponent(hourStart);
			starttimeLayout.addComponent(pointsLabel);
			starttimeLayout.setComponentAlignment(pointsLabel, Alignment.BOTTOM_CENTER);
			starttimeLayout.addComponent(minuteStart);
			
			hourEnd=new NativeSelect();
			hourEnd.setNullSelectionAllowed(false);
			minuteEnd=new NativeSelect();
			minuteEnd.setNullSelectionAllowed(false);
			
			endtimeLayout.addComponent(hourEnd);
			endtimeLayout.addComponent(pointsLabel2);
			endtimeLayout.setComponentAlignment(pointsLabel2, Alignment.BOTTOM_CENTER);
			endtimeLayout.addComponent(minuteEnd);

			teachers=new ComboBox(locale.getString("TEACHERSMANAGERLAYOUT_TEACHERSPANEL_CAPTION"));
			teachers.setSizeFull();
			teachers.setImmediate(true);
			teachers.setRequired(true);
			teachers.setInvalidAllowed(false);
			teachers.setNullSelectionAllowed(false);
			teachers.setFilteringMode(ComboBox.FILTERINGMODE_CONTAINS);
			teachers.setReadThrough(true);
			
			
			subjects=new ComboBox(locale.getString("SUBJECTSMANAGERLAYOUT_SUBJECTSPANEL_CAPTION"));
			subjects.setSizeFull();
			subjects.setImmediate(true);
			subjects.setRequired(true);
			subjects.setInvalidAllowed(false);
			subjects.setNullSelectionAllowed(false);
			subjects.setFilteringMode(ComboBox.FILTERINGMODE_CONTAINS);
			subjects.setReadThrough(true);
			
			groups=new ComboBox(locale.getString("GROUPMANAGERLAYOUT_GROUPSPANEL_CAPTION"));
			groups.setSizeFull();
			groups.setNullSelectionAllowed(false);
			groups.setImmediate(true);
			groups.setRequired(true);
			groups.setInvalidAllowed(false);
			groups.setFilteringMode(ComboBox.FILTERINGMODE_CONTAINS);
			groups.setReadThrough(true);	
	
			terms=new ComboBox(locale.getString("TERMMANAGERLAYOUT_TERMSPANEL_CAPTION"));
			terms.setSizeFull();
			terms.setNullSelectionAllowed(false);
			terms.setImmediate(true);
			terms.setRequired(true);
			terms.setFilteringMode(ComboBox.FILTERINGMODE_CONTAINS);
			terms.setReadThrough(true);	
	
			
			gl.addComponent(dayofweek,         0,0,9,0);
			gl.addComponent(labelStart,        0,1,3,1);
			gl.addComponent(starttimeLayout,   4,1,9,1);
			gl.addComponent(labelEnd,          0,2,3,2);
			gl.addComponent(endtimeLayout,     4,2,9,2);

			gl.addComponent(teachers,          0,4,9,4);
			gl.addComponent(subjects,          0,5,9,5);
			gl.addComponent(groups,            0,6,9,6);
			gl.addComponent(terms,             0,7,9,7);

			gl.setComponentAlignment(labelStart, Alignment.MIDDLE_CENTER);
			gl.setComponentAlignment(labelEnd, Alignment.MIDDLE_CENTER);
			gl.setSpacing(true);
			addComponent(gl);
		}

		public Boolean isEmptyTeachers(){
			if (teachers.size()==0){return true;}
			return false;
		}
		public Boolean isEmptyTerms(){
			if (terms.size()==0){return true;}
			return false;
		}
		public Boolean isEmptySubjects(){
			if (subjects.size()==0){return true;}
			return false;
		}
		public Boolean isEmptyGroups(){
			if (groups.size()==0){return true;}
			return false;
		}
		private void loadTeachers(){
			//Load Combo Teachers
			Integer firstItem=null;
			Collection<Teacher> collection = storage.allTeachers();
			IndexedContainer ict=new IndexedContainer();
			ict.addContainerProperty("fullname", String.class, null);
			Iterator<Teacher> it =collection.iterator();
			Item itemTeacher;
			while (it.hasNext()){
				Teacher teacher=it.next();
				if (firstItem==null){firstItem=teacher.getID();}
				itemTeacher=ict.addItem(teacher.getID());
				itemTeacher.getItemProperty("fullname").setValue(teacher.getFullname());
			}
			teachers.setContainerDataSource(ict);
			teachers.setItemCaptionPropertyId("fullname");
			teachers.setValue(firstItem);
		}
		private void loadTerms(){
			Integer firstItem=null;
			Collection<Term> collection = storage.allTerms();
			IndexedContainer ic=new IndexedContainer();
			ic.addContainerProperty("nameterm", String.class, null);
			Iterator<Term> it =collection.iterator();
			Item itemTerm;
			while (it.hasNext()){
				Term term=it.next();
				if (firstItem==null){firstItem= term.getID();}
				itemTerm=ic.addItem(term.getID());
				itemTerm.getItemProperty("nameterm").setValue(term.getName());
			}
			terms.setContainerDataSource(ic);
			terms.setItemCaptionPropertyId("nameterm");
			terms.setValue(firstItem);
		}
		private void loadSubjects(){
			Collection<Subject> collection = storage.allSubjects();
			Integer firstItem=null;
			IndexedContainer ic=new IndexedContainer();
			ic.addContainerProperty("name", String.class, null);
			Iterator<Subject> it =collection.iterator();
			Item itemSubject;
			while (it.hasNext()){
				Subject subject=it.next();
				if (firstItem==null){firstItem=subject.getID();};
				itemSubject=ic.addItem(subject.getID());
				itemSubject.getItemProperty("name").setValue(subject.getName());
			}
			subjects.setContainerDataSource(ic);
			subjects.setItemCaptionPropertyId("name");
			subjects.setValue(firstItem);
		}
		private void loadDaysofWeek(){
			String days[]={locale.getString("CLASSBLOCKMANAGERLAYOUT_DAYOFWEEK_SUNDAY"),
					locale.getString("CLASSBLOCKMANAGERLAYOUT_DAYOFWEEK_MONDAY"),
					locale.getString("CLASSBLOCKMANAGERLAYOUT_DAYOFWEEK_TUESDAY"),
					locale.getString("CLASSBLOCKMANAGERLAYOUT_DAYOFWEEK_WEDNESDAY"),
					locale.getString("CLASSBLOCKMANAGERLAYOUT_DAYOFWEEK_THURSDAY"),
					locale.getString("CLASSBLOCKMANAGERLAYOUT_DAYOFWEEK_FRIDAY"),
					locale.getString("CLASSBLOCKMANAGERLAYOUT_DAYOFWEEK_SATURDAY")};
			IndexedContainer icdaysofweek=new IndexedContainer();
			icdaysofweek.addContainerProperty("dayofweek", String.class, null);
			Item dayofweekItem;
			
			for (int i=1;i<8;i++){
				dayofweekItem=icdaysofweek.addItem(i);
				dayofweekItem.getItemProperty("dayofweek").setValue(days[i-1]);
			}
			dayofweek.setContainerDataSource(icdaysofweek);
			dayofweek.setItemCaptionPropertyId("dayofweek");
			dayofweek.setValue(1);
		}
		private void loadGroups(){
			Collection<Group> collection = storage.allGroups();
			Integer firstItem=null;
			IndexedContainer ic=new IndexedContainer();
			ic.addContainerProperty("namegroup", String.class, null);
			Iterator<Group> it =collection.iterator();
			Item itemGroup;
			while (it.hasNext()){
				Group group=it.next();
				if (firstItem==null){firstItem=group.getID();};
				itemGroup=ic.addItem(group.getID());
				itemGroup.getItemProperty("namegroup").setValue(group.getName());
			}
			groups.setContainerDataSource(ic);
			groups.setItemCaptionPropertyId("namegroup");
			groups.setValue(firstItem);
		}
		public void setHourStart(Time hour){
			//Combo hours
			String hourString=String.valueOf(hour.getHours());
			if (hourString.length()==1){hourString="0"+hourString;}
			hourStart.setValue(hourString);
			
			//Combo minutes
			String minuteString=String.valueOf(hour.getMinutes());
			if (minuteString.length()==1){minuteString="0"+minuteString;}
			minuteStart.setValue(minuteString);
		}
		public void setHourEnd(Time hour){
			//Combo hours
			String hourString=String.valueOf(hour.getHours());
			if (hourString.length()==1){hourString="0"+hourString;}
			hourEnd.setValue(hourString);
			
			//Combo minutes
			String minuteString=String.valueOf(hour.getMinutes());
			if (minuteString.length()==1){minuteString="0"+minuteString;}
			minuteEnd.setValue(minuteString);
		}
		public void setTeacher(int idteacher){
			teachers.setValue(idteacher);
		}
		public void setTerm(int idterm){
			terms.setValue(idterm);
		}
		public void setGroup(int idgroup){
			groups.setValue(idgroup);
		}
		public void setSubject(int idsubject){
			subjects.setValue(idsubject);
		}
		public void setDayofweek(Integer idDay){	
			dayofweek.setValue(idDay);
		}
		public Integer getSelectedTeacher(){
			Integer result =(Integer) teachers.getValue();
			return result;
		}
		public Integer getSelectedDayofweek(){
			Integer result =(Integer) dayofweek.getValue();
			return result;
		}
		public Integer getSelectTerm(){
			Integer result =(Integer) terms.getValue();
			return result;
		}
		public Integer getSelectedGroup(){
			Integer result =(Integer) groups.getValue();
			return result;
		}
		public Integer getSelectedSubject(){
			Integer result =(Integer) subjects.getValue();
			return result;
		}
		
		
		private void loadHours(){
			for (int i=0;i<24;i++){
				String timeString= String.valueOf(i);
				if (timeString.length()==1){ timeString="0"+timeString;}
				hourStart.addItem(timeString);
				hourEnd.addItem(timeString);
			}
			hourStart.setValue("00");
			hourEnd.setValue("00");
		}
		private void loadMinutes(){
			
			for (int i=0;i<60;i++){
				String timeString= String.valueOf(i);
				if (timeString.length()==1){ timeString="0"+timeString;}
				minuteStart.addItem(timeString);
				minuteEnd.addItem(timeString);
			}
			minuteStart.setValue("00");
			minuteEnd.setValue("00");
		}
		
		
		
		public Time getTimeStart(){
			String hour=hourStart.getValue() + ":" + minuteStart.getValue()+ ":00";
			Time result = Time.valueOf(hour);
			return result;
		}
		public Time getTimeEnd(){
			String hour=hourEnd.getValue() + ":" + minuteEnd.getValue()+ ":00";

			Time result = Time.valueOf(hour);

			return result;
		} 
		

	}

	
	/**
	 * Es la classe ImportClassblock en el qual s'importarà blocs lectius des d'un fitxer csv. Utilizarà un constructor de Classblock per llegir  les capceleres i també els camps dels csv.
	 * @see Classblock
	 * @see DatabaseStorage
	 * @see Upload
	 * @see CSVUploader
	 */
	private class ImportClassblockWindow extends Window implements Upload.SucceededListener, Upload.FailedListener{
		
		// Nou objecte de tipus CSVUploader.
		private CSVUploader csvuploader = new CSVUploader(Classblock.class.getConstructor(String.class, String.class, String.class, String.class, String.class), Classblock.getBasicProperties());
		
		// Nou objecte de tipus upload.
		private Upload upload=new Upload(locale.getString("GROUPMANAGERLAYOUT_UPLOAD_CAPTION"),csvuploader);
		
		/**
		 * Constructor de la ventana de importación de csv. Crea un objeto Upload y lo conecta con el CSVUploader.
		 * @throws SecurityException
		 * @throws NoSuchMethodException
		 */

		public ImportClassblockWindow() throws SecurityException, NoSuchMethodException {
			
			// Creem el missatge de notificació
			String alert = "<p><br/>Asegúrese de que la estructura de los registros del CSV sea la siguiente:</p>" + 
			"<table border=\"1\" align=\"center\">" +
			"<tr>" + "<th> Day </th>" + "<th> Period </th>" + "<th> Students Sets </th>" + "<th> Subject </th>" + 
			"<th> Teachers </th>" + "</tr>" + "<tr>" + "<th> \"Dilluns\"</th>" + "<th> \"08:00-09:00\"</th>" +
			"<th> \"group1\"</th>" + "<th> \"subject1\"</th>" + "<th> \"professor1 apellidos1\"</th>" +
			"</tr>" + "</table>";
			csvuploader.setAlert(alert);			
			
			upload.setButtonCaption(locale.getString("GROUPMANAGERLAYOUT_UPLOADBUTTON_CAPTION"));
			upload.addListener((Upload.FinishedListener)csvuploader);
			upload.addListener((Upload.SucceededListener)this);
			upload.addListener((Upload.FailedListener)this);
			
			// Afegim el botó upload.
			addComponent(upload);			
			
			// Creem el botó cancelar.
			Button cancelButton = new Button(locale.getString("GROUPMANAGERLAYOUT_BUTTONCANCEL_CAPTION"));

			// Controlem el event de cancelar per tancar la finestra d'importar.
			cancelButton.addListener(new Button.ClickListener() {
				
				@Override
				public void buttonClick(ClickEvent event) {
					close();
				}
			});
			
			// Afegim característiques de disseny i un horitzontal layout i l'afegim a la pantalla d'importar. 
			cancelButton.setIcon(new ThemeResource("img/12-em-cross.png"));
			HorizontalLayout hl = new HorizontalLayout();
			hl.setSpacing(true);
			hl.setMargin(true);
			hl.addComponent(cancelButton);
    		addComponent(hl);
    		
    		// Afegim un verticalLayout i li donem criteris de disseny a la pantalla d'importar.
    		VerticalLayout layout = (VerticalLayout) getContent();
    		layout.setSpacing(true);
    		layout.setMargin(true);
    		layout.setSizeUndefined(); // We want the window to has automatic size !!!
    		
    		// fem que la finestra sigui modal.
    		setModal(true);
    		setResizable(false);
		}

		/**
		 * Evento de carga correcta. Muestra una notificación con el número de registros correctos y fallidos.
		 */
		@Override
		public void uploadSucceeded(SucceededEvent event) {
			
			// Actualitzem la taula de classblocks.
			refreshClassblocks();
			close();
			System.out.println("Bloques lectivos cargados !!!");
			
			// Creem un missatge que després d'import blocslectius mostris quand blocslectius s'han carregat correctament i quans no.
			String finishMessage = locale.getString("GROUPMANAGERLAYOUT_UPLOADSUCCESS_TAGMESSAGE")+" "
				+ csvuploader.successfulRecordsCounter + " "
				+ locale.getString("GROUPMANAGERLAYOUT_UPLOADSUCCESSOK_MESSAGE")
				+ locale.getString("GROUPMANAGERLAYOUT_UPLOADSUCCESS_TAGMESSAGE")+" "
				+ csvuploader.failedRecordsCounter + " "
				+ locale.getString("GROUPMANAGERLAYOUT_UPLOADSUCCESSNOK_MESSAGE");
		
			// Afegim el misstage de notificació dels blocslectius carregats.
			Label finishLabel = csvuploader.getLabelFromString(finishMessage);
			AttendancemanagerApplication.adminWindow.showNotification("CARGA FINALIZADA",(String) finishLabel.getValue(), Notification.TYPE_TRAY_NOTIFICATION);
		}
		
		/**
		 * Evento de carga fallida. Si la carga falla, muestra una alerta de error.
		 */
		@Override
		public void uploadFailed(FailedEvent event) {
			System.out.println("Ohhhh, no he podido insertar los bloques lectivos !!!");
			String errorMessage = locale.getString("GROUPMANAGERLAYOUT_UPLOADFAILED_ERRORSTRING");
			Label errorLabel = csvuploader.getLabelFromString(errorMessage);
			AttendancemanagerApplication.adminWindow.showNotification("ERROR",(String) errorLabel.getValue(), Notification.TYPE_ERROR_MESSAGE);
		}
		
		
		/**
		 * Actualitza la base de dades storage
		 */
		public void refreshClassblocks() {

			storage.refreshTable(storage.getClassblocks(), classblocksTable);
		}
	}
	
/**
 * Aquesta classe mostrarà quan es seleccioni un professor els seus blocslectius
 * @see DatabaseStorage
 */
private class SearchClassblockWindow extends Window {
		
		// Creem un texfield per buscar per nom del professor i una taula per mostrar els blocslectius.
		public TextField buscar=new TextField(locale.getString("CLASSBLOCKMANAGERLAYOUT_TEACHER_SEARCH"));
		public Table table = new Table();

		/**
		 * És la classe constructora de SearchClassblockWindow.
		 * Buscarà per el nom complet del professor y mostrara els seus blocslectius enmagatzemats a storage en una taula. 
		 */
		public SearchClassblockWindow() {
			
			// Afegim un label amb una notoficació d'error y dos botons per aceptar i cancelar.
			final Label notF = new Label(locale.getString("CLASSBLOCKMANAGERLAYOUT_TEACHER_NOT_FOUND"), Notification.TYPE_ERROR_MESSAGE);
			Button okButton = new Button(locale.getString("GROUPMANAGERLAYOUT_BUTTONOK_CAPTION"));
			Button cancelButton = new Button(locale.getString("GROUPMANAGERLAYOUT_BUTTONCANCEL_CAPTION"));
			
			// Controlem el event de buscar que quan retorni blocslectius mostri una taula o si no torna res llavors que mostri un missatge.
			okButton.addListener(new Button.ClickListener() {		
				@Override
				public void buttonClick(ClickEvent event) {
					String teacher = buscar.getValue().toString();
					QueryContainer qc = storage.getClassblocks(teacher);
					table.setContainerDataSource(qc);
					if (qc.size() == 0) {
						notF.setVisible(true);
						table.setVisible(false);
					}
					else  {
						notF.setVisible(false);
						table.setVisible(true);
					}
					
				}		
			});
			
			// Controlem el event de cancelar per tancar la finestra.
			cancelButton.addListener(new Button.ClickListener() {		
				@Override
				public void buttonClick(ClickEvent event) {
					close();
				}
			});
			
			// Afegim una imatge al botons de aceptar i cancelar.
			okButton.setIcon(new ThemeResource("img/12-em-check.png"));
			cancelButton.setIcon(new ThemeResource("img/12-em-cross.png"));
			
			// Creem un horitzontalLayout i afegim els botons aceptar i cancelar. 
			HorizontalLayout hl = new HorizontalLayout();
			hl.setSpacing(true);
			hl.addComponent(okButton);
			hl.addComponent(cancelButton);
			
			// Afegim el horitzontalLayout, el texfield, la notoficació i la taula a la finestra de buscar.
			addComponent(buscar);
			addComponent(hl);
			addComponent(notF);
			addComponent(table);
			
			// Fem que quan s'obri la finestra de buscar només mostri el texfield i l'horitzontalLayout i fem la finestra modal.
			notF.setVisible(false);
			table.setVisible(false);
			setModal(true);
    		setResizable(false);
    		setWidth("33%");
    		setHeight("80%");
		}
	}
}